/**
 * Farbtastic Color Picker 1.2
 *  2008 Steven Wittens
 * Modified by Scott Ellis (2012)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA	02110-1301	USA
 */

var byId = function(id) { return document.getElementById(id); };
farbtastic = function(container, callback, initCol) {
	// Store farbtastic object
	var fb = this;

	// Insert markup
	byId(container).innerHTML = '<div class="farbtastic"><div class="color"></div><div class="wheel"></div><div class="overlay"></div><div class="h-marker marker"></div><div class="sl-marker marker"></div></div>';
	fb.col = byId(container).childNodes[0].childNodes[0];
	fb.wheel = byId(container).childNodes[0].childNodes[1];
	fb.hMarker = byId(container).childNodes[0].childNodes[3];
	fb.slMarker = byId(container).childNodes[0].childNodes[4];
	// Dimensions
	fb.radius = 84;
	fb.square = 100;
	fb.width = 194;

	fb.isIE = false;
	if (navigator.appName == 'Microsoft Internet Explorer') {
		var ua = navigator.userAgent;
		var re = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})");
		if (re.exec(ua) != null) {
			rv = parseFloat(RegExp.$1);
		}
		if (rv <= 8.0) {
			fb.isIE = true;
		}
	}
	fb.eventHook = function(ele, event, func, remove) {
		if (remove) {
			if (fb.isIE) {
				ele.detachEvent("on" + event, func);
			} else {
				ele.removeEventListener(event, func, false);
			}
		} else {
			if (fb.isIE) {
				ele.attachEvent("on" + event, func);
			} else {
				ele.addEventListener(event, func, false);
			}
		}
	}

	/**
	 * Link to the given element(s) or callback.
	 */
	fb.linkTo = function (callback) {
		// Unbind previous nodes
		if (typeof fb.callback == 'object') {
			fb.eventHook(fb.callback, 'keyup', fb.updateValue, true);
		}

		// Reset color
		fb.color = null;

		// Bind callback or elements
		if (typeof callback == 'function') {
			fb.callback = callback;
		}
		else if (typeof callback == 'object' || typeof callback == 'string') {
			fb.callback = byId(callback);
			fb.eventHook(fb.callback, 'keyup', fb.updateValue);
			if (fb.callback.value) {
				fb.setColor(fb.callback.value);
			}
		}
		return this;
	}
	fb.updateValue = function (event) {
		if (this.value && this.value != fb.color) {
			fb.setColor(this.value);
		}
	}

	/**
	 * Change color with HTML syntax #123456
	 */
	fb.setColor = function (color) {
		var unpack = fb.unpack(color);
		if (fb.color != color && unpack) {
			fb.color = color;
			fb.rgb = unpack;
			fb.hsl = fb.RGBToHSL(fb.rgb);
			fb.updateDisplay();
		}
		return this;
	}

	/**
	 * Change color with HSL triplet [0..1, 0..1, 0..1]
	 */
	fb.setHSL = function (hsl) {
		fb.hsl = hsl;
		fb.rgb = fb.HSLToRGB(hsl);
		fb.color = fb.pack(fb.rgb);
		fb.updateDisplay();
		return this;
	}

	/////////////////////////////////////////////////////

	/**
	 * Retrieve the coordinates of the given event relative to the center
	 * of the widget.
	 */
	fb.widgetCoords = function (event) {
		var x, y;
		var el = event.target || event.srcElement;
		var reference = fb.wheel;

		if (typeof event.offsetX != 'undefined') {
			// Use offset coordinates and find common offsetParent
			var pos = { x: event.offsetX, y: event.offsetY };

			// Send the coordinates upwards through the offsetParent chain.
			var e = el;
			while (e) {
				e.mouseX = pos.x;
				e.mouseY = pos.y;
				pos.x += e.offsetLeft;
				pos.y += e.offsetTop;
				e = e.offsetParent;
			}

			// Look for the coordinates starting from the wheel widget.
			var e = reference;
			var offset = { x: 0, y: 0 }
			while (e) {
				if (typeof e.mouseX != 'undefined') {
					x = e.mouseX - offset.x;
					y = e.mouseY - offset.y;
					break;
				}
				offset.x += e.offsetLeft;
				offset.y += e.offsetTop;
				e = e.offsetParent;
			}

			// Reset stored coordinates
			e = el;
			while (e) {
				e.mouseX = undefined;
				e.mouseY = undefined;
				e = e.offsetParent;
			}
		}
		else {
			// Use absolute coordinates
			var pos = fb.absolutePosition(reference);
			x = (event.pageX - (document.body.scrollLeft || document.documentElement.scrollLeft) || event.clientX) - pos.x;
			y = (event.pageY - (document.body.scrollTop || document.documentElement.scrollTop) || event.clientY) - pos.y;
		}
		// Subtract distance to middle
		return { x: x - fb.width / 2, y: y - fb.width / 2 };
	}

	/**
	 * Mousedown handler
	 */
	fb.mousedown = function (event) {
		// Capture mouse
		if (!document.dragging) {
			fb.eventHook(document, 'mousemove', fb.mousemove);
			fb.eventHook(document, 'mouseup', fb.mouseup);
			document.dragging = true;
		}

		// Check which area is being dragged
		var pos = fb.widgetCoords(event);
		fb.circleDrag = Math.max(Math.abs(pos.x), Math.abs(pos.y)) * 2 > fb.square;

		// Process
		fb.mousemove(event);
		event.stopPropagation();
		event.preventDefault();
		return false;
	}

	/**
	 * Mousemove handler
	 */
	fb.mousemove = function (event) {
		// Get coordinates relative to color picker center
		var pos = fb.widgetCoords(event);

		// Set new HSL parameters
		if (fb.circleDrag) {
			var hue = Math.atan2(pos.x, -pos.y) / 6.28;
			if (hue < 0) hue += 1;
			fb.setHSL([hue, fb.hsl[1], fb.hsl[2]]);
		}
		else {
			var sat = Math.max(0, Math.min(1, -(pos.x / fb.square) + .5));
			var lum = Math.max(0, Math.min(1, -(pos.y / fb.square) + .5));
			fb.setHSL([fb.hsl[0], sat, lum]);
		}
		return false;
	}

	/**
	 * Mouseup handler
	 */
	fb.mouseup = function () {
		// Uncapture mouse
		fb.eventHook(document, 'mousemove', fb.mousemove, true);
		fb.eventHook(document, 'mouseup', fb.mouseup, true);
		document.dragging = false;
	}

	/**
	 * Update the markers and styles
	 */
	fb.updateDisplay = function () {
		// Markers
		var angle = fb.hsl[0] * 6.28;
		fb.hMarker.style.left = Math.round(Math.sin(angle) * fb.radius + fb.width / 2) + 'px';
		fb.hMarker.style.top = Math.round(-Math.cos(angle) * fb.radius + fb.width / 2) + 'px';

		fb.slMarker.style.left = Math.round(fb.square * (.5 - fb.hsl[1]) + fb.width / 2) + 'px';
		fb.slMarker.style.top = Math.round(fb.square * (.5 - fb.hsl[2]) + fb.width / 2) + 'px';

		// Saturation/Luminance gradient
		fb.col.style.backgroundColor = fb.pack(fb.HSLToRGB([fb.hsl[0], 1, 0.5]));

		// Linked elements or callback
		if (typeof fb.callback == 'object') {
			// Set background/foreground color
			fb.callback.style.backgroundColor = fb.color;
			fb.callback.style.color = fb.hsl[2] > 0.5 ? '#000' : '#fff';

			// Change linked value -- made lame
			if (fb.callback.value && fb.callback.value != fb.color) {
				fb.callback.value = fb.color;
			}
		}
		else if (typeof fb.callback == 'function') {
			fb.callback.call(fb, fb.color);
		}
	}

	/**
	 * Get absolute position of element
	 */
	fb.absolutePosition = function (el) {
		var r = { x: el.offsetLeft, y: el.offsetTop };
		// Resolve relative to offsetParent
		if (el.offsetParent) {
			var tmp = fb.absolutePosition(el.offsetParent);
			r.x += tmp.x;
			r.y += tmp.y;
		}
		return r;
	};

	/* Various color utility functions */
	fb.pack = function (rgb) {
		var r = Math.round(rgb[0] * 255);
		var g = Math.round(rgb[1] * 255);
		var b = Math.round(rgb[2] * 255);
		return '#' + (r < 16 ? '0' : '') + r.toString(16) +
					 (g < 16 ? '0' : '') + g.toString(16) +
					 (b < 16 ? '0' : '') + b.toString(16);
	}

	fb.unpack = function (color) {
		if (color.length == 7) {
			return [parseInt('0x' + color.substring(1, 3)) / 255,
				parseInt('0x' + color.substring(3, 5)) / 255,
				parseInt('0x' + color.substring(5, 7)) / 255];
		}
		else if (color.length == 4) {
			return [parseInt('0x' + color.substring(1, 2)) / 15,
				parseInt('0x' + color.substring(2, 3)) / 15,
				parseInt('0x' + color.substring(3, 4)) / 15];
		}
	}

	fb.HSLToRGB = function (hsl) {
		var m1, m2, r, g, b;
		var h = hsl[0], s = hsl[1], l = hsl[2];
		m2 = (l <= 0.5) ? l * (s + 1) : l + s - l*s;
		m1 = l * 2 - m2;
		return [this.hueToRGB(m1, m2, h+0.33333),
				this.hueToRGB(m1, m2, h),
				this.hueToRGB(m1, m2, h-0.33333)];
	}

	fb.hueToRGB = function (m1, m2, h) {
		h = (h < 0) ? h + 1 : ((h > 1) ? h - 1 : h);
		if (h * 6 < 1) return m1 + (m2 - m1) * h * 6;
		if (h * 2 < 1) return m2;
		if (h * 3 < 2) return m1 + (m2 - m1) * (0.66666 - h) * 6;
		return m1;
	}

	fb.RGBToHSL = function (rgb) {
		var min, max, delta, h, s, l;
		var r = rgb[0], g = rgb[1], b = rgb[2];
		min = Math.min(r, Math.min(g, b));
		max = Math.max(r, Math.max(g, b));
		delta = max - min;
		l = (min + max) / 2;
		s = 0;
		if (l > 0 && l < 1) {
			s = delta / (l < 0.5 ? (2 * l) : (2 - 2 * l));
		}
		h = 0;
		if (delta > 0) {
			if (max == r && max != g) h += (g - b) / delta;
			if (max == g && max != b) h += (2 + (b - r) / delta);
			if (max == b && max != r) h += (4 + (r - g) / delta);
			h /= 6;
		}
		return [h, s, l];
	}

	// Install mousedown handler (the others are set on the document on-demand)
	for (i = 0; i < byId(container).childNodes[0].childNodes.length; ++i) {
		fb.eventHook(byId(container).childNodes[0].childNodes[i], 'mousedown', fb.mousedown);
	}

		// Init color
	fb.setColor(initCol || '#000000');
	
	// Set linked elements/callback
	if (callback) {
		fb.linkTo(callback);
	}
	return fb;
}